import os
import sys
import shutil

from nose.tools import assert_equal
from nose.tools import assert_list_equal
from nose.tools import with_setup

from linchpin.cli import LinchpinCli
from linchpin.cli.context import LinchpinCliContext
from linchpin.tests.mockdata.contextdata import ContextData


def test_context_create():

    lpc = LinchpinCliContext()
    assert_equal(isinstance(lpc, LinchpinCliContext), True)


def setup_load_config():

    """
    Perform setup of ContextData() object, and run get_temp_filename()
    """

    global cd
    global provider
    global config_path
    global config_data

    provider = 'dummy'

    cd = ContextData()
    # cd.load_config_data(provider)
    cd.load_config_data()
    cd.create_config()
    config_path = cd.get_temp_filename()
    config_data = cd.cfg_data
    cd.write_config_file(config_path)


def setup_lp_fetch_env():
    global lpc
    global lpcli

    global target
    global pinfile
    global mockpath

    base_path = '{0}'.format(os.path.dirname(
        os.path.realpath(__file__))).rstrip('/')
    lib_path = os.path.realpath(os.path.join(base_path, os.pardir))

    setup_load_config()

    pinfile = 'PinFile'
    mock_path = '{0}/{1}/{2}'.format(lib_path, 'mockdata', provider)

    lpc = LinchpinCliContext()
    lpc.load_config(lpconfig=config_path)
    lpc.load_global_evars()
    lpc.setup_logging()
    lpc.workspace = '/tmp/workspace/'
    mockpath = os.path.realpath(mock_path)

    if not os.path.exists(lpc.workspace):
        os.mkdir(lpc.workspace)

    lpcli = LinchpinCli(lpc)


@with_setup(setup_lp_fetch_env)
def test_fetch_local():

    src_path = os.path.join(mockpath, 'fetch/ws1')
    src_uri = 'file://{0}'.format(src_path)
    lpcli.lp_fetch(src_uri, root=None, fetch_type='workspace')

    src_list = os.listdir(src_path)
    dest_list = os.listdir(lpc.workspace)
    src_list.sort()
    dest_list.sort()
    shutil.rmtree(lpc.workspace)
    shutil.rmtree(os.path.join(os.path.expanduser('~'), '.cache/linchpin/'))
    assert_list_equal(src_list, dest_list)


@with_setup(setup_lp_fetch_env)
def test_fetch_git():

    src_url = 'https://github.com/agharibi/SampleLinchpinDirectory.git'
    lpcli.lp_fetch(src_url, root='ws1', fetch_type='topologies')

    src_list = ['topologies']
    dest_list = os.listdir(lpc.workspace)

    shutil.rmtree(lpc.workspace)
    shutil.rmtree(os.path.join(os.path.expanduser('~'), '.cache/linchpin/'))

    assert_list_equal(src_list, dest_list)


@with_setup(setup_lp_fetch_env)
def test_fetch_cache():

    src_url = 'https://github.com/agharibi/SampleLinchpinDirectory.git'
    lpcli.lp_fetch(src_url, root='ws1', fetch_type='topologies')

    cache_path = os.path.join(os.path.expanduser('~'), '.cache/linchpin/git')

    cache_dir_list = os.listdir(cache_path)
    shutil.rmtree(lpc.workspace)
    shutil.rmtree(os.path.join(os.path.expanduser('~'), '.cache/linchpin/'))

    assert_equal(1, len(cache_dir_list))


def main():

    setup_load_config()


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python

import StringIO

from InventoryFilter import InventoryFilter


class GCloudInventory(InventoryFilter):

    def get_host_ips(self, topo):
        host_public_ips = []
        for group in topo['gcloud_gce_res']:
            for instance in group['instance_data']:
                host_public_ips.append(instance['public_ip'])
        return host_public_ips

    def get_inventory(self, topo, layout):
        if len(topo['gcloud_gce_res']) == 0:
            return ""
        # get inventory hosts
        inven_hosts = self.get_host_ips(topo)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/env python

import StringIO

from InventoryFilter import InventoryFilter


class AWSInventory(InventoryFilter):

    def get_host_ips(self, topo):
        """
        Returns a list of hostnames or IP addresses for use in an Ansible
        inventory file, based on available data. Only a single hostname or IP
        address will be returned per instance, so as to avoid duplicate runs of
        Ansible on the same host via the generated inventory file.

        If an instance has a public IP attached, its hostname in DNS will be
        returned if available, and if not the public IP address will be used.
        For instances which have a private IP address for VPC use cases, the
        private IP address will be returned since private EC2 hostnames (e.g.
        ip-10-0-0-1.ec2.internal) will not typically be resolvable outside of
        AWS. For instances with both a public and private IP address, the
        public address is always returned instead of the private address.

        :param topo:
            linchpin AWS EC2 resource data
        """

        host_dns_ip = []
        for group in topo['aws_ec2_res']:
            for instance in group['instances']:
                if instance['public_dns_name']:
                    host_dns_ip.append(str(instance['public_dns_name']))
                elif instance['public_ip']:
                    host_dns_ip.append(str(instance['public_ip']))
                else:
                    host_dns_ip.append(str(instance['private_ip']))
        return host_dns_ip

    def get_inventory(self, topo, layout):

        if len(topo['aws_ec2_res']) == 0:
            return ""
        inven_hosts = self.get_host_ips(topo)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()
